@@ -38,7 +38,7 @@ public:
   /**
    * The name of the command as specified in CMakeList.txt.
    */
-  virtual const char* GetName() const { return "ctest_configure";}
+  virtual std::string GetName() const { return "ctest_configure";}
 
   cmTypeMacro(cmCTestConfigureCommand, cmCTestHandlerCommand);
 
