@@ -83,17 +83,13 @@ public:
    */
   virtual void Configure();
 
-  bool Compute();
-  virtual void AddExtraIDETargets() {}
+  virtual bool Compute();
 
   enum TargetTypes {
     AllTargets,
     ImportedOnly
   };
 
-  void CreateImportedGenerationObjects(cmMakefile* mf,
-                             std::vector<std::string> const& targets,
-                             std::vector<cmGeneratorTarget const*>& exports);
   void CreateGenerationObjects(TargetTypes targetTypes = AllTargets);
 
   /**
@@ -250,9 +246,7 @@ public:
   cmTarget* FindTarget(const std::string& name,
                        bool excludeAliases = false) const;
 
-  cmGeneratorTarget* FindGeneratorTarget(const std::string& name) const;
-
-  void AddAlias(const std::string& name, const std::string& tgtName);
+  void AddAlias(const std::string& name, cmTarget *tgt);
   bool IsAlias(const std::string& name) const;
 
   /** Determine if a name resolves to a framework on disk or a built target
@@ -278,8 +272,7 @@ public:
   std::set<std::string> const& GetDirectoryContent(std::string const& dir,
                                                    bool needDisk = true);
 
-  void IndexTarget(cmTarget* t);
-  void IndexGeneratorTarget(cmGeneratorTarget* gt);
+  void AddTarget(cmTarget* t);
 
   static bool IsReservedTarget(std::string const& name);
 
@@ -306,6 +299,14 @@ public:
   TargetDependSet const& GetTargetDirectDepends(
       const cmGeneratorTarget* target);
 
+  /** Get per-target generator information.  */
+  cmGeneratorTarget* GetGeneratorTarget(cmTarget const*) const;
+
+  void AddGeneratorTarget(cmTarget* t, cmGeneratorTarget* gt)
+  {
+    this->GeneratorTargets[t] = gt;
+  }
+
   const std::map<std::string, std::vector<cmLocalGenerator*> >& GetProjectMap()
                                                const {return this->ProjectMap;}
 
@@ -352,16 +353,15 @@ public:
   void CreateEvaluationSourceFiles(std::string const& config) const;
 
   void SetFilenameTargetDepends(cmSourceFile* sf,
-                                std::set<const cmGeneratorTarget*> tgts);
-  const std::set<const cmGeneratorTarget*>&
+                                std::set<cmTarget const*> tgts);
+  std::set<cmTarget const*> const&
   GetFilenameTargetDepends(cmSourceFile* sf) const;
 
 #if defined(CMAKE_BUILD_WITH_CMAKE)
   cmFileLockPool& GetFileLockPool() { return FileLockPool; }
 #endif
 
-  bool GetConfigureDoneCMP0026() const
-  { return this->ConfigureDoneCMP0026AndCMP0024; }
+  bool GetConfigureDoneCMP0026() const { return this->ConfigureDoneCMP0026; }
 
   std::string MakeSilentFlag;
 protected:
@@ -371,7 +371,7 @@ protected:
   void GetTargetSets(TargetDependSet& projectTargets,
                              TargetDependSet& originalTargets,
                              cmLocalGenerator* root, GeneratorVector const&);
-  bool IsRootOnlyTarget(cmGeneratorTarget* target) const;
+  bool IsRootOnlyTarget(cmTarget* target) const;
   void AddTargetDepends(const cmGeneratorTarget* target,
                         TargetDependSet& projectTargets);
   void SetLanguageEnabledFlag(const std::string& l, cmMakefile* mf);
@@ -384,7 +384,7 @@ protected:
 
   virtual bool CheckALLOW_DUPLICATE_CUSTOM_TARGETS() const;
 
-  std::vector<const cmGeneratorTarget*> CreateQtAutoGeneratorsTargets();
+  std::vector<cmTarget const*> CreateQtAutoGeneratorsTargets();
 
   std::string SelectMakeProgram(const std::string& makeProgram,
                                 const std::string& makeDefault = "") const;
@@ -420,38 +420,24 @@ protected:
   std::map<std::string, cmExportBuildFileGenerator*> BuildExportSets;
   std::map<std::string, cmExportBuildFileGenerator*> BuildExportExportSets;
 
-  std::map<std::string, std::string> AliasTargets;
-
-  cmTarget* FindTargetImpl(std::string const& name) const;
-
-  cmGeneratorTarget* FindGeneratorTargetImpl(std::string const& name) const;
-  cmGeneratorTarget*
-  FindImportedGeneratorTargetImpl(std::string const& name) const;
+  // All targets in the entire project.
+#if defined(CMAKE_BUILD_WITH_CMAKE)
+#ifdef CMake_HAVE_CXX11_UNORDERED_MAP
+  typedef std::unordered_map<std::string, cmTarget*> TargetMap;
+#else
+  typedef cmsys::hash_map<std::string, cmTarget*> TargetMap;
+#endif
+#else
+  typedef std::map<std::string,cmTarget *> TargetMap;
+#endif
+  TargetMap TotalTargets;
+  TargetMap AliasTargets;
+  TargetMap ImportedTargets;
 
   const char* GetPredefinedTargetsFolder();
   virtual bool UseFolderProperty();
 
 private:
-
-#if defined(CMAKE_BUILD_WITH_CMAKE)
-# ifdef CMake_HAVE_CXX11_UNORDERED_MAP
-  typedef std::unordered_map<std::string, cmTarget*> TargetMap;
-  typedef std::unordered_map<std::string, cmGeneratorTarget*>
-    GeneratorTargetMap;
-# else
-  typedef cmsys::hash_map<std::string, cmTarget*> TargetMap;
-  typedef cmsys::hash_map<std::string, cmGeneratorTarget*> GeneratorTargetMap;
-# endif
-#else
-  typedef std::map<std::string,cmTarget *> TargetMap;
-  typedef std::map<std::string,cmGeneratorTarget *> GeneratorTargetMap;
-#endif
-  // Map efficiently from target name to cmTarget instance.
-  // Do not use this structure for looping over all targets.
-  // It contains both normal and globally visible imported targets.
-  TargetMap TargetSearchIndex;
-  GeneratorTargetMap GeneratorTargetSearchIndex;
-
   cmMakefile* TryCompileOuterMakefile;
   // If you add a new map here, make sure it is copied
   // in EnableLanguagesFromGenerator
@@ -471,7 +457,7 @@ private:
   void WriteRuleHashes(std::string const& pfile);
 
   void WriteSummary();
-  void WriteSummary(cmGeneratorTarget* target);
+  void WriteSummary(cmTarget* target);
   void FinalizeTargetCompileInfo();
 
   virtual void ForceLinkerLanguages();
@@ -481,8 +467,6 @@ private:
   void CheckCompilerIdCompatibility(cmMakefile* mf,
                                     std::string const& lang) const;
 
-  void ComputeBuildFileGenerators();
-
   cmExternalMakefileProjectGenerator* ExtraGenerator;
 
   // track files replaced during a Generate
@@ -492,10 +476,10 @@ private:
   typedef std::map<cmGeneratorTarget const*, TargetDependSet> TargetDependMap;
   TargetDependMap TargetDependencies;
 
+  // Per-target generator information.
+  cmGeneratorTargetsType GeneratorTargets;
   friend class cmake;
-  void CreateGeneratorTargets(TargetTypes targetTypes, cmMakefile* mf,
-                   cmLocalGenerator* lg,
-                   std::map<cmTarget*, cmGeneratorTarget*> const& importedMap);
+  void CreateGeneratorTargets(TargetTypes targetTypes, cmLocalGenerator* lg);
   void CreateGeneratorTargets(TargetTypes targetTypes);
 
   void ClearGeneratorMembers();
@@ -520,7 +504,7 @@ private:
   // track targets to issue CMP0042 warning for.
   std::set<std::string> CMP0042WarnTargets;
 
-  mutable std::map<cmSourceFile*, std::set<cmGeneratorTarget const*> >
+  mutable std::map<cmSourceFile*, std::set<cmTarget const*> >
   FilenameTargetDepends;
 
 #if defined(CMAKE_BUILD_WITH_CMAKE)
@@ -535,7 +519,7 @@ protected:
   bool ForceUnixPaths;
   bool ToolSupportsColor;
   bool InstallTargetEnabled;
-  bool ConfigureDoneCMP0026AndCMP0024;
+  bool ConfigureDoneCMP0026;
 };
 
 #endif
