@@ -16,20 +16,31 @@
 #endif
 
 #include KWSYS_HEADER(SystemTools.hxx)
-#include KWSYS_HEADER(ios/iostream)
 
 // Work-around CMake dependency scanning limitation.  This must
 // duplicate the above list of headers.
 #if 0
 # include "SystemTools.hxx.in"
-# include "kwsys_ios_iostream.h.in"
 #endif
 
 // Include with <> instead of "" to avoid getting any in-source copy
 // left on disk.
 #include <testSystemTools.h>
 
+#include <iostream>
+#include <sstream>
 #include <string.h> /* strcmp */
+#if defined(_WIN32) && !defined(__CYGWIN__)
+# include <io.h> /* _umask (MSVC) / umask (Borland) */
+# ifdef _MSC_VER
+#  define umask _umask // Note this is still umask on Borland
+# endif
+#endif
+#include <sys/stat.h> /* umask (POSIX), _S_I* constants (Windows) */
+// Visual C++ does not define mode_t (note that Borland does, however).
+#if defined( _MSC_VER )
+typedef unsigned short mode_t;
+#endif
 
 //----------------------------------------------------------------------------
 static const char* toUnixPaths[][2] =
@@ -52,17 +63,17 @@ static const char* toUnixPaths[][2] =
     {0, 0}
 };
 
-static bool CheckConvertToUnixSlashes(kwsys_stl::string input,
-                                      kwsys_stl::string output)
+static bool CheckConvertToUnixSlashes(std::string input,
+                                      std::string output)
 {
-  kwsys_stl::string result = input;
+  std::string result = input;
   kwsys::SystemTools::ConvertToUnixSlashes(result);
   if ( result != output )
     {
-    kwsys_ios::cerr
+    std::cerr
       << "Problem with ConvertToUnixSlashes - input: " << input
       << " output: " << result << " expected: " << output
-      << kwsys_ios::endl;
+      << std::endl;
     return false;
     }
   return true;
@@ -76,19 +87,19 @@ static const char* checkEscapeChars[][4] =
   {0, 0, 0, 0}
 };
 
-static bool CheckEscapeChars(kwsys_stl::string input,
+static bool CheckEscapeChars(std::string input,
                              const char *chars_to_escape,
                              char escape_char,
-                             kwsys_stl::string output)
+                             std::string output)
 {
-  kwsys_stl::string result = kwsys::SystemTools::EscapeChars(
+  std::string result = kwsys::SystemTools::EscapeChars(
     input.c_str(), chars_to_escape, escape_char);
   if (result != output)
     {
-    kwsys_ios::cerr
+    std::cerr
       << "Problem with CheckEscapeChars - input: " << input
       << " output: " << result << " expected: " << output
-      << kwsys_ios::endl;
+      << std::endl;
     return false;
     }
   return true;
@@ -98,78 +109,229 @@ static bool CheckEscapeChars(kwsys_stl::string input,
 static bool CheckFileOperations()
 {
   bool res = true;
-  const kwsys_stl::string testBinFile(TEST_SYSTEMTOOLS_SOURCE_DIR
+  const std::string testNonExistingFile(TEST_SYSTEMTOOLS_SOURCE_DIR
+    "/testSystemToolsNonExistingFile");
+  const std::string testDotFile(TEST_SYSTEMTOOLS_SOURCE_DIR
+    "/.");
+  const std::string testBinFile(TEST_SYSTEMTOOLS_SOURCE_DIR
     "/testSystemTools.bin");
-  const kwsys_stl::string testTxtFile(TEST_SYSTEMTOOLS_SOURCE_DIR
+  const std::string testTxtFile(TEST_SYSTEMTOOLS_SOURCE_DIR
     "/testSystemTools.cxx");
-  const kwsys_stl::string testNewDir(TEST_SYSTEMTOOLS_BINARY_DIR
+  const std::string testNewDir(TEST_SYSTEMTOOLS_BINARY_DIR
     "/testSystemToolsNewDir");
-  const kwsys_stl::string testNewFile(testNewDir + "/testNewFile.txt");
+  const std::string testNewFile(testNewDir + "/testNewFile.txt");
+
+  if (kwsys::SystemTools::DetectFileType(testNonExistingFile.c_str()) !=
+      kwsys::SystemTools::FileTypeUnknown)
+    {
+    std::cerr
+      << "Problem with DetectFileType - failed to detect type of: "
+      << testNonExistingFile << std::endl;
+    res = false;
+    }
+
+  if (kwsys::SystemTools::DetectFileType(testDotFile.c_str()) !=
+      kwsys::SystemTools::FileTypeUnknown)
+    {
+    std::cerr
+      << "Problem with DetectFileType - failed to detect type of: "
+      << testDotFile << std::endl;
+    res = false;
+    }
 
   if (kwsys::SystemTools::DetectFileType(testBinFile.c_str()) !=
       kwsys::SystemTools::FileTypeBinary)
     {
-    kwsys_ios::cerr
+    std::cerr
       << "Problem with DetectFileType - failed to detect type of: "
-      << testBinFile << kwsys_ios::endl;
+      << testBinFile << std::endl;
     res = false;
     }
 
   if (kwsys::SystemTools::DetectFileType(testTxtFile.c_str()) !=
       kwsys::SystemTools::FileTypeText)
     {
-    kwsys_ios::cerr
+    std::cerr
       << "Problem with DetectFileType - failed to detect type of: "
-      << testTxtFile << kwsys_ios::endl;
+      << testTxtFile << std::endl;
     res = false;
     }
 
   if (kwsys::SystemTools::FileLength(testBinFile) != 766)
     {
-    kwsys_ios::cerr
+    std::cerr
       << "Problem with FileLength - incorrect length for: "
-      << testBinFile << kwsys_ios::endl;
+      << testBinFile << std::endl;
     res = false;
     }
 
   if (!kwsys::SystemTools::MakeDirectory(testNewDir))
     {
-    kwsys_ios::cerr
+    std::cerr
       << "Problem with MakeDirectory for: "
-      << testNewDir << kwsys_ios::endl;
+      << testNewDir << std::endl;
     res = false;
     }
 
   if (!kwsys::SystemTools::Touch(testNewFile.c_str(), true))
     {
-    kwsys_ios::cerr
+    std::cerr
       << "Problem with Touch for: "
-      << testNewFile << kwsys_ios::endl;
+      << testNewFile << std::endl;
     res = false;
     }
 
+  // Reset umask
+#if defined(_WIN32) && !defined(__CYGWIN__)
+  // NOTE:  Windows doesn't support toggling _S_IREAD.
+  mode_t fullMask = _S_IWRITE;
+#else
+  // On a normal POSIX platform, we can toggle all permissions.
+  mode_t fullMask = S_IRWXU | S_IRWXG | S_IRWXO;
+#endif
+  mode_t orig_umask = umask(fullMask);
+
+  // Test file permissions without umask
+  mode_t origPerm, thisPerm;
+  if (!kwsys::SystemTools::GetPermissions(testNewFile, origPerm))
+    {
+    std::cerr
+      << "Problem with GetPermissions (1) for: "
+      << testNewFile << std::endl;
+    res = false;
+    }
+
+  if (!kwsys::SystemTools::SetPermissions(testNewFile, 0))
+    {
+    std::cerr
+      << "Problem with SetPermissions (1) for: "
+      << testNewFile << std::endl;
+    res = false;
+    }
+
+  if (!kwsys::SystemTools::GetPermissions(testNewFile, thisPerm))
+    {
+    std::cerr
+      << "Problem with GetPermissions (2) for: "
+      << testNewFile << std::endl;
+    res = false;
+    }
+
+  if ((thisPerm & fullMask) != 0)
+    {
+    std::cerr
+      << "SetPermissions failed to set permissions (1) for: "
+      << testNewFile << ": actual = " << thisPerm << "; expected = "
+      << 0 << std::endl;
+    res = false;
+    }
+
+  // While we're at it, check proper TestFileAccess functionality.
+  if (kwsys::SystemTools::TestFileAccess(testNewFile,
+                                         kwsys::TEST_FILE_WRITE))
+    {
+    std::cerr
+      << "TestFileAccess incorrectly indicated that this is a writable file:"
+      << testNewFile << std::endl;
+    res = false;
+    }
+
+  if (!kwsys::SystemTools::TestFileAccess(testNewFile,
+                                          kwsys::TEST_FILE_OK))
+    {
+    std::cerr
+      << "TestFileAccess incorrectly indicated that this file does not exist:"
+      << testNewFile << std::endl;
+    res = false;
+    }
+
+  // Test restoring/setting full permissions.
+  if (!kwsys::SystemTools::SetPermissions(testNewFile, fullMask))
+    {
+    std::cerr
+      << "Problem with SetPermissions (2) for: "
+      << testNewFile << std::endl;
+    res = false;
+    }
+
+  if (!kwsys::SystemTools::GetPermissions(testNewFile, thisPerm))
+    {
+    std::cerr
+      << "Problem with GetPermissions (3) for: "
+      << testNewFile << std::endl;
+    res = false;
+    }
+
+  if ((thisPerm & fullMask) != fullMask)
+    {
+    std::cerr
+      << "SetPermissions failed to set permissions (2) for: "
+      << testNewFile << ": actual = " << thisPerm << "; expected = "
+      << fullMask << std::endl;
+    res = false;
+    }
+
+  // Test setting file permissions while honoring umask
+  if (!kwsys::SystemTools::SetPermissions(testNewFile, fullMask, true))
+    {
+    std::cerr
+      << "Problem with SetPermissions (3) for: "
+      << testNewFile << std::endl;
+    res = false;
+    }
+
+  if (!kwsys::SystemTools::GetPermissions(testNewFile, thisPerm))
+    {
+    std::cerr
+      << "Problem with GetPermissions (4) for: "
+      << testNewFile << std::endl;
+    res = false;
+    }
+
+  if ((thisPerm & fullMask) != 0)
+    {
+    std::cerr
+      << "SetPermissions failed to honor umask for: "
+      << testNewFile << ": actual = " << thisPerm << "; expected = "
+      << 0 << std::endl;
+    res = false;
+    }
+
+  // Restore umask
+  umask(orig_umask);
+
+  // Restore file permissions
+  if (!kwsys::SystemTools::SetPermissions(testNewFile, origPerm))
+    {
+    std::cerr
+      << "Problem with SetPermissions (4) for: "
+      << testNewFile << std::endl;
+    res = false;
+    }
+
+  // Remove the test file
   if (!kwsys::SystemTools::RemoveFile(testNewFile))
     {
-    kwsys_ios::cerr
+    std::cerr
       << "Problem with RemoveFile: "
-      << testNewFile << kwsys_ios::endl;
+      << testNewFile << std::endl;
     res = false;
     }
 
-  kwsys_stl::string const testFileMissing(testNewDir + "/testMissingFile.txt");
+  std::string const testFileMissing(testNewDir + "/testMissingFile.txt");
   if (!kwsys::SystemTools::RemoveFile(testFileMissing))
     {
     std::string const& msg = kwsys::SystemTools::GetLastSystemError();
-    kwsys_ios::cerr <<
+    std::cerr <<
       "RemoveFile(\"" << testFileMissing << "\") failed: " << msg << "\n";
     res = false;
     }
 
-  kwsys_stl::string const testFileMissingDir(testNewDir + "/missing/file.txt");
+  std::string const testFileMissingDir(testNewDir + "/missing/file.txt");
   if (!kwsys::SystemTools::RemoveFile(testFileMissingDir))
     {
     std::string const& msg = kwsys::SystemTools::GetLastSystemError();
-    kwsys_ios::cerr <<
+    std::cerr <<
       "RemoveFile(\"" << testFileMissingDir << "\") failed: " << msg << "\n";
     res = false;
     }
@@ -177,9 +339,9 @@ static bool CheckFileOperations()
   kwsys::SystemTools::Touch(testNewFile.c_str(), true);
   if (!kwsys::SystemTools::RemoveADirectory(testNewDir))
     {
-    kwsys_ios::cerr
+    std::cerr
       << "Problem with RemoveADirectory for: "
-      << testNewDir << kwsys_ios::endl;
+      << testNewDir << std::endl;
     res = false;
     }
 
@@ -187,14 +349,14 @@ static bool CheckFileOperations()
   // Perform the same file and directory creation and deletion tests but
   // with paths > 256 characters in length.
 
-  const kwsys_stl::string testNewLongDir(
+  const std::string testNewLongDir(
     TEST_SYSTEMTOOLS_BINARY_DIR "/"
     "012345678901234567890123456789012345678901234567890123456789"
     "012345678901234567890123456789012345678901234567890123456789"
     "012345678901234567890123456789012345678901234567890123456789"
     "012345678901234567890123456789012345678901234567890123456789"
     "01234567890123");
-  const kwsys_stl::string testNewLongFile(testNewLongDir + "/"
+  const std::string testNewLongFile(testNewLongDir + "/"
     "012345678901234567890123456789012345678901234567890123456789"
     "012345678901234567890123456789012345678901234567890123456789"
     "012345678901234567890123456789012345678901234567890123456789"
@@ -203,34 +365,34 @@ static bool CheckFileOperations()
 
   if (!kwsys::SystemTools::MakeDirectory(testNewLongDir))
     {
-    kwsys_ios::cerr
+    std::cerr
       << "Problem with MakeDirectory for: "
-      << testNewLongDir << kwsys_ios::endl;
+      << testNewLongDir << std::endl;
     res = false;
     }
 
   if (!kwsys::SystemTools::Touch(testNewLongFile.c_str(), true))
     {
-    kwsys_ios::cerr
+    std::cerr
       << "Problem with Touch for: "
-      << testNewLongFile << kwsys_ios::endl;
+      << testNewLongFile << std::endl;
     res = false;
     }
 
   if (!kwsys::SystemTools::RemoveFile(testNewLongFile))
     {
-    kwsys_ios::cerr
+    std::cerr
       << "Problem with RemoveFile: "
-      << testNewLongFile << kwsys_ios::endl;
+      << testNewLongFile << std::endl;
     res = false;
     }
 
   kwsys::SystemTools::Touch(testNewLongFile.c_str(), true);
   if (!kwsys::SystemTools::RemoveADirectory(testNewLongDir))
     {
-    kwsys_ios::cerr
+    std::cerr
       << "Problem with RemoveADirectory for: "
-      << testNewLongDir << kwsys_ios::endl;
+      << testNewLongDir << std::endl;
     res = false;
     }
 #endif
@@ -243,84 +405,84 @@ static bool CheckStringOperations()
 {
   bool res = true;
 
-  kwsys_stl::string test = "mary had a little lamb.";
+  std::string test = "mary had a little lamb.";
   if (kwsys::SystemTools::CapitalizedWords(test) != "Mary Had A Little Lamb.")
     {
-    kwsys_ios::cerr
+    std::cerr
       << "Problem with CapitalizedWords "
-      << '"' << test << '"' << kwsys_ios::endl;
-    res = false;    
+      << '"' << test << '"' << std::endl;
+    res = false;
     }
 
   test = "Mary Had A Little Lamb.";
-  if (kwsys::SystemTools::UnCapitalizedWords(test) != 
+  if (kwsys::SystemTools::UnCapitalizedWords(test) !=
       "mary had a little lamb.")
     {
-    kwsys_ios::cerr
+    std::cerr
       << "Problem with UnCapitalizedWords "
-      << '"' << test << '"' << kwsys_ios::endl;
-    res = false;    
+      << '"' << test << '"' << std::endl;
+    res = false;
     }
 
   test = "MaryHadTheLittleLamb.";
-  if (kwsys::SystemTools::AddSpaceBetweenCapitalizedWords(test) != 
+  if (kwsys::SystemTools::AddSpaceBetweenCapitalizedWords(test) !=
       "Mary Had The Little Lamb.")
     {
-    kwsys_ios::cerr
+    std::cerr
       << "Problem with AddSpaceBetweenCapitalizedWords "
-      << '"' << test << '"' << kwsys_ios::endl;
-    res = false;    
+      << '"' << test << '"' << std::endl;
+    res = false;
     }
 
-  char * cres = 
+  char * cres =
     kwsys::SystemTools::AppendStrings("Mary Had A"," Little Lamb.");
   if (strcmp(cres,"Mary Had A Little Lamb."))
     {
-    kwsys_ios::cerr
+    std::cerr
       << "Problem with AppendStrings "
-      << "\"Mary Had A\" \" Little Lamb.\"" << kwsys_ios::endl;
-    res = false;    
+      << "\"Mary Had A\" \" Little Lamb.\"" << std::endl;
+    res = false;
     }
   delete [] cres;
 
-  cres = 
+  cres =
     kwsys::SystemTools::AppendStrings("Mary Had"," A ","Little Lamb.");
   if (strcmp(cres,"Mary Had A Little Lamb."))
     {
-    kwsys_ios::cerr
+    std::cerr
       << "Problem with AppendStrings "
-      << "\"Mary Had\" \" A \" \"Little Lamb.\"" << kwsys_ios::endl;
-    res = false;    
+      << "\"Mary Had\" \" A \" \"Little Lamb.\"" << std::endl;
+    res = false;
     }
   delete [] cres;
 
   if (kwsys::SystemTools::CountChar("Mary Had A Little Lamb.",'a') != 3)
     {
-    kwsys_ios::cerr
+    std::cerr
       << "Problem with CountChar "
-      << "\"Mary Had A Little Lamb.\"" << kwsys_ios::endl;
-    res = false;    
+      << "\"Mary Had A Little Lamb.\"" << std::endl;
+    res = false;
     }
 
-  cres = 
+  cres =
     kwsys::SystemTools::RemoveChars("Mary Had A Little Lamb.","aeiou");
   if (strcmp(cres,"Mry Hd A Lttl Lmb."))
     {
-    kwsys_ios::cerr
+    std::cerr
       << "Problem with RemoveChars "
-      << "\"Mary Had A Little Lamb.\"" << kwsys_ios::endl;
-    res = false;    
+      << "\"Mary Had A Little Lamb.\"" << std::endl;
+    res = false;
     }
   delete [] cres;
 
-  cres = 
+  cres =
     kwsys::SystemTools::RemoveCharsButUpperHex("Mary Had A Little Lamb.");
   if (strcmp(cres,"A"))
     {
-    kwsys_ios::cerr
+    std::cerr
       << "Problem with RemoveCharsButUpperHex "
-      << "\"Mary Had A Little Lamb.\"" << kwsys_ios::endl;
-    res = false;    
+      << "\"Mary Had A Little Lamb.\"" << std::endl;
+    res = false;
     }
   delete [] cres;
 
@@ -329,59 +491,59 @@ static bool CheckStringOperations()
   kwsys::SystemTools::ReplaceChars(cres2,"aeiou",'X');
   if (strcmp(cres2,"MXry HXd A LXttlX LXmb."))
     {
-    kwsys_ios::cerr
+    std::cerr
       << "Problem with ReplaceChars "
-      << "\"Mary Had A Little Lamb.\"" << kwsys_ios::endl;
-    res = false;    
+      << "\"Mary Had A Little Lamb.\"" << std::endl;
+    res = false;
     }
   delete [] cres2;
 
   if (!kwsys::SystemTools::StringStartsWith("Mary Had A Little Lamb.",
                                             "Mary "))
     {
-    kwsys_ios::cerr
+    std::cerr
       << "Problem with StringStartsWith "
-      << "\"Mary Had A Little Lamb.\"" << kwsys_ios::endl;
-    res = false;    
+      << "\"Mary Had A Little Lamb.\"" << std::endl;
+    res = false;
     }
 
   if (!kwsys::SystemTools::StringEndsWith("Mary Had A Little Lamb.",
                                           " Lamb."))
     {
-    kwsys_ios::cerr
+    std::cerr
       << "Problem with StringEndsWith "
-      << "\"Mary Had A Little Lamb.\"" << kwsys_ios::endl;
-    res = false;    
+      << "\"Mary Had A Little Lamb.\"" << std::endl;
+    res = false;
     }
 
   cres = kwsys::SystemTools::DuplicateString("Mary Had A Little Lamb.");
   if (strcmp(cres,"Mary Had A Little Lamb."))
     {
-    kwsys_ios::cerr
+    std::cerr
       << "Problem with DuplicateString "
-      << "\"Mary Had A Little Lamb.\"" << kwsys_ios::endl;
-    res = false;    
+      << "\"Mary Had A Little Lamb.\"" << std::endl;
+    res = false;
     }
   delete [] cres;
 
   test = "Mary Had A Little Lamb.";
-  if (kwsys::SystemTools::CropString(test,13) != 
+  if (kwsys::SystemTools::CropString(test,13) !=
       "Mary ...Lamb.")
     {
-    kwsys_ios::cerr
+    std::cerr
       << "Problem with CropString "
-      << "\"Mary Had A Little Lamb.\"" << kwsys_ios::endl;
-    res = false;    
+      << "\"Mary Had A Little Lamb.\"" << std::endl;
+    res = false;
     }
 
-  kwsys_stl::vector<kwsys_stl::string> lines;
+  std::vector<std::string> lines;
   kwsys::SystemTools::Split("Mary Had A Little Lamb.",lines,' ');
   if (lines[0] != "Mary" || lines[1] != "Had" ||
       lines[2] != "A" || lines[3] != "Little" || lines[4] != "Lamb.")
     {
-    kwsys_ios::cerr
+    std::cerr
       << "Problem with Split "
-      << "\"Mary Had A Little Lamb.\"" << kwsys_ios::endl;
+      << "\"Mary Had A Little Lamb.\"" << std::endl;
     res = false;
     }
 
@@ -390,10 +552,10 @@ static bool CheckStringOperations()
       ("L:\\Local Mojo\\Hex Power Pack\\Iffy Voodoo") !=
       L"\\\\?\\L:\\Local Mojo\\Hex Power Pack\\Iffy Voodoo")
     {
-    kwsys_ios::cerr
+    std::cerr
       << "Problem with ConvertToWindowsExtendedPath "
       << "\"L:\\Local Mojo\\Hex Power Pack\\Iffy Voodoo\""
-      << kwsys_ios::endl;
+      << std::endl;
     res = false;
     }
 
@@ -401,10 +563,10 @@ static bool CheckStringOperations()
       ("L:/Local Mojo/Hex Power Pack/Iffy Voodoo") !=
       L"\\\\?\\L:\\Local Mojo\\Hex Power Pack\\Iffy Voodoo")
     {
-    kwsys_ios::cerr
+    std::cerr
       << "Problem with ConvertToWindowsExtendedPath "
       << "\"L:/Local Mojo/Hex Power Pack/Iffy Voodoo\""
-      << kwsys_ios::endl;
+      << std::endl;
     res = false;
     }
 
@@ -412,10 +574,10 @@ static bool CheckStringOperations()
       ("\\\\Foo\\Local Mojo\\Hex Power Pack\\Iffy Voodoo") !=
       L"\\\\?\\UNC\\Foo\\Local Mojo\\Hex Power Pack\\Iffy Voodoo")
     {
-    kwsys_ios::cerr
+    std::cerr
       << "Problem with ConvertToWindowsExtendedPath "
       << "\"\\\\Foo\\Local Mojo\\Hex Power Pack\\Iffy Voodoo\""
-      << kwsys_ios::endl;
+      << std::endl;
     res = false;
     }
 
@@ -423,106 +585,106 @@ static bool CheckStringOperations()
       ("//Foo/Local Mojo/Hex Power Pack/Iffy Voodoo") !=
       L"\\\\?\\UNC\\Foo\\Local Mojo\\Hex Power Pack\\Iffy Voodoo")
     {
-    kwsys_ios::cerr
+    std::cerr
       << "Problem with ConvertToWindowsExtendedPath "
       << "\"//Foo/Local Mojo/Hex Power Pack/Iffy Voodoo\""
-      << kwsys_ios::endl;
+      << std::endl;
     res = false;
     }
 
   if (kwsys::SystemTools::ConvertToWindowsExtendedPath("//") !=
       L"//")
     {
-    kwsys_ios::cerr
+    std::cerr
       << "Problem with ConvertToWindowsExtendedPath "
       << "\"//\""
-      << kwsys_ios::endl;
+      << std::endl;
     res = false;
     }
 
   if (kwsys::SystemTools::ConvertToWindowsExtendedPath("\\\\.\\") !=
       L"\\\\.\\")
     {
-    kwsys_ios::cerr
+    std::cerr
       << "Problem with ConvertToWindowsExtendedPath "
       << "\"\\\\.\\\""
-      << kwsys_ios::endl;
+      << std::endl;
     res = false;
     }
 
   if (kwsys::SystemTools::ConvertToWindowsExtendedPath("\\\\.\\X") !=
       L"\\\\.\\X")
     {
-    kwsys_ios::cerr
+    std::cerr
       << "Problem with ConvertToWindowsExtendedPath "
       << "\"\\\\.\\X\""
-      << kwsys_ios::endl;
+      << std::endl;
     res = false;
     }
 
   if (kwsys::SystemTools::ConvertToWindowsExtendedPath("\\\\.\\X:") !=
       L"\\\\?\\X:")
     {
-    kwsys_ios::cerr
+    std::cerr
       << "Problem with ConvertToWindowsExtendedPath "
       << "\"\\\\.\\X:\""
-      << kwsys_ios::endl;
+      << std::endl;
     res = false;
     }
 
   if (kwsys::SystemTools::ConvertToWindowsExtendedPath("\\\\.\\X:\\") !=
       L"\\\\?\\X:\\")
     {
-    kwsys_ios::cerr
+    std::cerr
       << "Problem with ConvertToWindowsExtendedPath "
       << "\"\\\\.\\X:\\\""
-      << kwsys_ios::endl;
+      << std::endl;
     res = false;
     }
 
   if (kwsys::SystemTools::ConvertToWindowsExtendedPath("NUL") !=
       L"\\\\.\\NUL")
     {
-    kwsys_ios::cerr
+    std::cerr
       << "Problem with ConvertToWindowsExtendedPath "
       << "\"NUL\""
-      << kwsys_ios::endl;
+      << std::endl;
     res = false;
     }
 
 #endif
 
   if (kwsys::SystemTools::ConvertToWindowsOutputPath
-      ("L://Local Mojo/Hex Power Pack/Iffy Voodoo") != 
+      ("L://Local Mojo/Hex Power Pack/Iffy Voodoo") !=
       "\"L:\\Local Mojo\\Hex Power Pack\\Iffy Voodoo\"")
     {
-    kwsys_ios::cerr
+    std::cerr
       << "Problem with ConvertToWindowsOutputPath "
       << "\"L://Local Mojo/Hex Power Pack/Iffy Voodoo\""
-      << kwsys_ios::endl;
-    res = false;    
+      << std::endl;
+    res = false;
     }
-  
+
   if (kwsys::SystemTools::ConvertToWindowsOutputPath
-      ("//grayson/Local Mojo/Hex Power Pack/Iffy Voodoo") != 
+      ("//grayson/Local Mojo/Hex Power Pack/Iffy Voodoo") !=
       "\"\\\\grayson\\Local Mojo\\Hex Power Pack\\Iffy Voodoo\"")
     {
-    kwsys_ios::cerr
+    std::cerr
       << "Problem with ConvertToWindowsOutputPath "
       << "\"//grayson/Local Mojo/Hex Power Pack/Iffy Voodoo\""
-      << kwsys_ios::endl;
-    res = false;    
+      << std::endl;
+    res = false;
     }
 
   if (kwsys::SystemTools::ConvertToUnixOutputPath
-      ("//Local Mojo/Hex Power Pack/Iffy Voodoo") != 
+      ("//Local Mojo/Hex Power Pack/Iffy Voodoo") !=
       "//Local\\ Mojo/Hex\\ Power\\ Pack/Iffy\\ Voodoo")
     {
-    kwsys_ios::cerr
+    std::cerr
       << "Problem with ConvertToUnixOutputPath "
       << "\"//Local Mojo/Hex Power Pack/Iffy Voodoo\""
-      << kwsys_ios::endl;
-    res = false;    
+      << std::endl;
+    res = false;
     }
 
   return res;
@@ -530,20 +692,20 @@ static bool CheckStringOperations()
 
 //----------------------------------------------------------------------------
 
-static bool CheckPutEnv(const kwsys_stl::string& env, const char* name, const char* value)
+static bool CheckPutEnv(const std::string& env, const char* name, const char* value)
 {
   if(!kwsys::SystemTools::PutEnv(env))
     {
-    kwsys_ios::cerr << "PutEnv(\"" << env
-                    << "\") failed!" << kwsys_ios::endl;
+    std::cerr << "PutEnv(\"" << env
+                    << "\") failed!" << std::endl;
     return false;
     }
   const char* v = kwsys::SystemTools::GetEnv(name);
   v = v? v : "(null)";
   if(strcmp(v, value) != 0)
     {
-    kwsys_ios::cerr << "GetEnv(\"" << name << "\") returned \""
-                    << v << "\", not \"" << value << "\"!" << kwsys_ios::endl;
+    std::cerr << "GetEnv(\"" << name << "\") returned \""
+                    << v << "\", not \"" << value << "\"!" << std::endl;
     return false;
     }
   return true;
@@ -553,14 +715,14 @@ static bool CheckUnPutEnv(const char* env, const char* name)
 {
   if(!kwsys::SystemTools::UnPutEnv(env))
     {
-    kwsys_ios::cerr << "UnPutEnv(\"" << env << "\") failed!"
-                    << kwsys_ios::endl;
+    std::cerr << "UnPutEnv(\"" << env << "\") failed!"
+                    << std::endl;
     return false;
     }
   if(const char* v = kwsys::SystemTools::GetEnv(name))
     {
-    kwsys_ios::cerr << "GetEnv(\"" << name << "\") returned \""
-                    << v << "\", not (null)!" << kwsys_ios::endl;
+    std::cerr << "GetEnv(\"" << name << "\") returned \""
+                    << v << "\", not (null)!" << std::endl;
     return false;
     }
   return true;
@@ -582,15 +744,15 @@ static bool CheckEnvironmentOperations()
 
 
 static bool CheckRelativePath(
-  const kwsys_stl::string& local,
-  const kwsys_stl::string& remote,
-  const kwsys_stl::string& expected)
+  const std::string& local,
+  const std::string& remote,
+  const std::string& expected)
 {
-  kwsys_stl::string result = kwsys::SystemTools::RelativePath(local, remote);
+  std::string result = kwsys::SystemTools::RelativePath(local, remote);
   if(expected != result)
     {
-    kwsys_ios::cerr << "RelativePath(" << local << ", " << remote
-      << ")  yielded " << result << " instead of " << expected << kwsys_ios::endl;
+    std::cerr << "RelativePath(" << local << ", " << remote
+      << ")  yielded " << result << " instead of " << expected << std::endl;
     return false;
     }
   return true;
@@ -608,14 +770,14 @@ static bool CheckRelativePaths()
 }
 
 static bool CheckCollapsePath(
-  const kwsys_stl::string& path,
-  const kwsys_stl::string& expected)
+  const std::string& path,
+  const std::string& expected)
 {
-  kwsys_stl::string result = kwsys::SystemTools::CollapseFullPath(path);
+  std::string result = kwsys::SystemTools::CollapseFullPath(path);
   if(expected != result)
     {
-    kwsys_ios::cerr << "CollapseFullPath(" << path
-      << ")  yielded " << result << " instead of " << expected << kwsys_ios::endl;
+    std::cerr << "CollapseFullPath(" << path
+      << ")  yielded " << result << " instead of " << expected << std::endl;
     return false;
     }
   return true;
@@ -629,6 +791,66 @@ static bool CheckCollapsePath()
   return res;
 }
 
+static std::string StringVectorToString(const std::vector<std::string>& vec)
+{
+  std::stringstream ss;
+  ss << "vector(";
+  for (std::vector<std::string>::const_iterator i = vec.begin();
+       i != vec.end(); ++i)
+    {
+    if (i != vec.begin())
+      {
+      ss << ", ";
+      }
+    ss << *i;
+    }
+  ss << ")";
+  return ss.str();
+}
+
+static bool CheckGetPath()
+{
+  const char* envName = "S";
+#ifdef _WIN32
+  const char* envValue = "C:\\Somewhere\\something;D:\\Temp";
+#else
+  const char* envValue = "/Somewhere/something:/tmp";
+#endif
+  const char* registryPath = "[HKEY_LOCAL_MACHINE\\SOFTWARE\\MyApp; MyKey]";
+
+  std::vector<std::string> originalPathes;
+  originalPathes.push_back(registryPath);
+
+  std::vector<std::string> expectedPathes;
+  expectedPathes.push_back(registryPath);
+#ifdef _WIN32
+  expectedPathes.push_back("C:/Somewhere/something");
+  expectedPathes.push_back("D:/Temp");
+#else
+  expectedPathes.push_back("/Somewhere/something");
+  expectedPathes.push_back("/tmp");
+#endif
+
+  bool res = true;
+  res &= CheckPutEnv(std::string(envName) + "=" + envValue, envName, envValue);
+
+  std::vector<std::string> pathes = originalPathes;
+  kwsys::SystemTools::GetPath(pathes, envName);
+
+  if (pathes != expectedPathes)
+    {
+    std::cerr <<
+      "GetPath(" << StringVectorToString(originalPathes) <<
+      ", " << envName << ")  yielded " << StringVectorToString(pathes) <<
+      " instead of " << StringVectorToString(expectedPathes) <<
+      std::endl;
+    res = false;
+    }
+
+  res &= CheckUnPutEnv(envName, envName);
+  return res;
+}
+
 //----------------------------------------------------------------------------
 int testSystemTools(int, char*[])
 {
@@ -641,7 +863,7 @@ int testSystemTools(int, char*[])
     }
 
   // Special check for ~
-  kwsys_stl::string output;
+  std::string output;
   if(kwsys::SystemTools::GetEnv("HOME", output))
     {
     output += "/foo bar/lala";
@@ -650,7 +872,7 @@ int testSystemTools(int, char*[])
 
   for (cc = 0; checkEscapeChars[cc][0]; cc ++ )
     {
-    res &= CheckEscapeChars(checkEscapeChars[cc][0], checkEscapeChars[cc][1], 
+    res &= CheckEscapeChars(checkEscapeChars[cc][0], checkEscapeChars[cc][1],
                             *checkEscapeChars[cc][2], checkEscapeChars[cc][3]);
     }
 
@@ -664,5 +886,7 @@ int testSystemTools(int, char*[])
 
   res &= CheckCollapsePath();
 
+  res &= CheckGetPath();
+
   return res ? 0 : 1;
 }
